
from typing import List


class PromptBook:
    @staticmethod
    def expand_requirements(question:str)->str:
        return f"""
you are a senior programmer below is what your client have asked you to do:
---
{question}
---

your current task is to list down all the clarifications you need from the client:
dont ask anything like budget or timelines and other business related stuff.

As your response will go to an automated parser, things to keep in mind all the time:
* write one questions per line
* don't write anything other than the questions.
* use emojis in the question to make them fun.
* Remember the client is a non technical person so ask question accordingly

Begin!
"""
    @staticmethod
    def plan_project(question:str, clarification:str)->str:
        return f"""
you are a senior programmer below is what your client have asked you to do:
---
{question}
---
here are some clarrification on the requirements
---
{clarification}
---
your current task is to design the folder and and files structure for this project
you should follow the below format to answer this question 

Plan : <your plan here> //tell us a very detailed, verbose plan or thought on how you want to proceed before writing the files  for example which framework you are going to use,dont forget to mention framewark related configuration files. Discuss project architecture so junior developer can understand very easily

As your response will go to an automated parser, things to keep in mind all the time:
* follow the exact format provided above without fail
* be detailed so junior developer can understand very easily
Begin!
"""
    @staticmethod
    def design_folder_structure(question:str,plan:str,clarifications:str)->str:
        return f"""
you are a senior programmer below is what your client have asked you to do:
---
{question}
---
here are some clarrification on the requirements
---
{clarifications}
---
below is the what you have already planed what to do:
---
{plan}
---
Now based on this write a json object to design file tree.

```
{{
    "root_dir_name": "name of the project",
    "files": [
        {{
        "name": "name of the file",
        "type":"file/directory",
        "files": [...] //repeat the same pattern if its a directory
        }}
    ]
}}
```
As your response will go to an automated parser, things to keep in mind all the time:
* follow the exact format provided above without fail
* only write the json nothing else, no expiation, no pretext 
* do not write the contents of the files that will be done by the next developer
Begin!
"""
    @staticmethod
    def prioritise_file_list(filelist:List[str])->List[str]:
        list_string='\n'.join(filelist)
        return f"""
you are a senior programmer. you going to write a api service.
below are the file list that you are going to complete in future.

reorder them in a way that most suitable based on how one will be dependent on other

{list_string}

As your response will go to an automated parser, things to keep in mind all the time:
* only write file list in order nothing else
* do not write any explanation
Begin!
        """
    @staticmethod
    def write_file(question,clarifications:str,plan:str,files_written:List[List[str]], file_path_to_write:str,file_paths:List[str])->str:
        file_with_conent="\n\n".join([f"File:{file_path}\nContent:\n{content}" for file_path,content in files_written])
        all_files_list="\n".join(file_paths)
        return f"""
you are a senior programmer below is what your client have asked you to do:
---
{question}
---
here are some clarrification on the requirements
---
{clarifications}
---
below is the what you have already planed what to do:
---
{plan}
---
Below are the full files list that already has been or will be written
--
{all_files_list}
--

you are now writing the code below are the files that already written with content as follows:
---
{file_with_conent}
---

now you are about to write content the following file:
{file_path_to_write}

As your response will go to an automated parser, things to keep in mind all the time:
* follow the exact format provided above without fail
* only write the file content, no expiation, no pretext.
* if the language support, add comments at steps, which expains what you are about to do, dont add comment if comment is not supported by the file type example json file
* keep in mind there wont be any additional files other then the full files list given above, only use files that are mentioned in that list
Begin!
"""





import sys
from typing import List
import os
from .prompts import PromptBook
from langchain.llms import OpenAI
from langchain.chat_models import ChatOpenAI
from langchain.prompts.chat import (
    ChatPromptTemplate,
    SystemMessagePromptTemplate,
    AIMessagePromptTemplate,
    HumanMessagePromptTemplate,
)
from langchain.schema import (
    AIMessage,
    HumanMessage,
    SystemMessage
)

import json

from .utils import Utilities

class Developer():
    def __init__(self, requirement:str,root_dir:str, openai_api_key:str):
        self.requirement=requirement
        self.root_dir=root_dir
        self.api_key=openai_api_key
        self.files_written=[]
        self.brain = ChatOpenAI(
            model="gpt-3.5-turbo",
            openai_api_key=self.api_key,
            temperature=0.1,
            streaming=False
            )
    

    def brain_storm(self,prompt:str,step_name="prompt")->str:
        messages = [
            SystemMessage(content="you are a senior software developer"),
            HumanMessage(content=prompt)
        ]
        aiMessage:AIMessage= self.brain(messages=messages)
        if not os.path.exists("./thoughts"):
            os.makedirs("./thoughts")
        if os.path.exists(f"./thoughts/{step_name}.md"):
            os.remove(f"./thoughts/{step_name}.md")
        Utilities.write_to_file(f"{prompt}\n\n{aiMessage.content}",f"./thoughts/{step_name}.md")
        return aiMessage.content
    
    def clear_doubts(self):
        prompt=PromptBook.expand_requirements(self.requirement)
        doubts=self.brain_storm(prompt,'clear-doubts')
        doubt_list:List[str]=doubts.split("\n")
        doubt_list=[doubt.strip() for doubt in doubt_list if doubt.strip()!=""]
        print("""
Hey there! 😄 It's Lazy Dev, your friendly neighborhood programmer, here to make your awesome project's dreams come true! 🎉 
But before I dive into coding magic, I have a few fun and important questions for you. 
So, grab a cup of coffee ☕️, sit back, and let's clarify some details, shall we? Here we go! 🚀
        """)
        answer_list=[]
        for doubt in doubt_list:
            answer = input(f"{doubt}\n>>")
            answer_list.append(answer)

        print("""
Thats all I need! 😄

Sit back and relax while I work my coding magic for you! ✨✨✨

🚀 I've got this! 🎉

Cheers! 👨‍💻
        """)
        return doubt_list,answer_list

    def plan_project(self):
        prompt=PromptBook.plan_project(self.requirement,self.clarifications)
        plannings:str=self.brain_storm(prompt,'plan-project')
        return plannings
    
    def generate_folder_structure(self):
        prompt=PromptBook.design_folder_structure(
            question=self.requirement,
            plan=self.plannings,
            clarifications=self.clarifications
            )
        retry_count=3
        while retry_count>0:
            try:

                folder_tree_str:str=self.brain_storm(prompt,"generate-filders")
                folder_tree:dict=json.loads(folder_tree_str)
                break
            except:
                print("Opps messed up the json format, let me try again")
                retry_count=retry_count-1
        
        if retry_count==0:
            print("Sorry I was not able to create the folder structure in json correct format, check my instructions and try to refine it sothat i can understan the task better")
            sys.exit()       
        self.root_folder_name, self.file_paths = Utilities.generate_files_and_folders(structure=folder_tree,root_dir=self.root_dir)


    def prioratize_files(self):
        prompt=PromptBook.prioritise_file_list(self.file_paths)
        retry_count=3
        while retry_count>0:
            try:
                file_paths_str=self.brain_storm(prompt,'prioratize_files') 
                break
            except:
                print("Opps messed up the json format, let me try again")
                retry_count=retry_count-1
        if retry_count==0:
            print("Sorry I was not able to create the file list in correct format, check my instructions and try to refine it sothat i can understan the task better")
            sys.exit()       
        self.file_paths=file_paths_str.split("\n")
    
    def write_file_content(self,file_path):
        prompt=PromptBook.write_file(
            question=self.requirement,
            clarifications=self.clarifications,
            plan=self.plannings,
            files_written=self.files_written,
            file_path_to_write=file_path,
            file_paths=self.file_paths
            )
        code=self.brain_storm(prompt,f'code-{file_path.split("/")[-1]}') 
        Utilities.write_to_file(code,file_path=file_path)
        self.files_written.append((
            file_path,
            code
        ))

    def develop(self):
        # clearing all doubts
        doubts,answers=self.clear_doubts()
        self.clarifications:str=""
        for i in range(len(doubts)):
            self.clarifications=f"{self.clarifications}\n\n{i+1}. {doubts[i]}\n Ans: {answers[i]}"

        # planning the project
        print("Planning...")
        self.plannings=self.plan_project()
        # creating files and folders for the project
        print("Creating files...")
        self.generate_folder_structure()
        self.prioratize_files()
        self.files_written=[]
        for file_path in self.file_paths:
            file_name=file_path.split("/")[-1]
            if(file_name.split(".")[-1] in ["png","jpg","jpeg","bimp"]):
                continue
            print(f"\nWriting Code for :{file_name}")
            self.write_file_content(file_path)




        


import os
from typing import List, Dict

class Utilities:
    @staticmethod
    def touch(path):
        with open(path, 'a'):
            os.utime(path, None)

    @staticmethod
    def generate_files_recursively(files:List[Dict],cur_dir:str):
        file_paths=[]
        for file in files:
            name=file['name']
            type=file['type']
            item_path=os.path.join(cur_dir,name)
            if type=="file":
                Utilities.touch(item_path)
                file_paths.append(item_path)
                print(f"Created File: {item_path}")
            else:
                if not os.path.exists(item_path):
                    os.makedirs(item_path)
                    print(f"Created Directory: {item_path}")
                subfiles:List[Dict]=file['files']
                sub_file_paths=Utilities.generate_files_recursively(subfiles,item_path)
                file_paths=file_paths+sub_file_paths
        return file_paths

    @staticmethod
    def generate_files_and_folders(structure:dict,root_dir:str):
        root_dir_name=structure['root_dir_name']
        cur_dir=os.path.join(root_dir,root_dir_name)
        root_dir_path=cur_dir
        if not os.path.exists(cur_dir):
            os.makedirs(cur_dir)
            print(f"Created Directory: {cur_dir}")
        files:List[Dict]=structure['files']
        file_paths=Utilities.generate_files_recursively(files=files,cur_dir=cur_dir)
        return root_dir_path, file_paths

    def write_to_file(content:str,file_path:str):
        # Open the file in write mode
        with open(file_path, "w") as file:
            # Write the content to the file
            file.write(content)



from setuptools import find_packages, setup

with open('README.md', 'r', encoding='utf-8') as f:
    readme = f.read()

setup(
    name="lazydev",
    version='0.0.5',
    packages=find_packages(),
    install_requires=[
        "langchain>=0.0.188",
        "openai>=0.27.7"
    ],
    entry_points={
        'console_scripts': [
            'lazydev = lazydev:run',
        ],
    },
    author='Anirban Kat',
    author_email='thecodacus@gmail.com',
    description='AI developer for lazy programmer',
    long_description=readme,  # Assign the contents of README.md to long_description
    long_description_content_type='text/markdown',  # Specify the type of long description
    url='https://github.com/thecodacus/lazy-dev',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
    # other relevant information

    # # You can specify package data if needed
    # package_data={
    #     'lazydev': ['*.txt', '*.csv'],  # Include any additional files here
    # },

    # # You can also specify additional non-package data files
    # data_files=[
    #     ('config', ['config.ini']),
    # ],

)


import argparse
from . import develop

# Create the top-level parser
parser = argparse.ArgumentParser()

# Create subparsers for the different subcommands
subparsers = parser.add_subparsers(title='Subcommands', dest='subcommand')

# Import and execute the subcommand modules
develop.setup_parser(subparsers)

# Parse the command-line arguments
args = parser.parse_args()

# Execute the appropriate subcommand
def run():
    if hasattr(args, 'func'):
        args.func(args)











from argparse import _SubParsersAction, ArgumentParser
import os
from dotenv import load_dotenv
from .modules.developer import Developer


def setup_parser(sub_parsers:_SubParsersAction):
    # putting import inside so that code can be moved to saperate files and we dont keep the import at top of this file
    # Create the argument parser
    parser = sub_parsers.add_parser(name='develop', help='develop command for new project creation')
    

    # Add the "--requirements" argument with the "-r" shortcut
    parser.add_argument('--requirement', '-r', required=True, type=str, help='The initial requirement')

    # Add the "--directory" argument with the "-d" shortcut
    parser.add_argument('--directory', '-d', default="./code", type=str, help='The directory path to put generated files')

    parser.add_argument('--model', '-m', default="gpt-3.5-turbo", type=str, help='GPT Mode to use. options: gpt-3.5-turbo, gpt-4')

    parser.set_defaults(func=run)

    return parser

def run(args):

    requirement = args.requirement
    directory= args.directory
    load_dotenv()
    api_key = os.environ.get('OPENAI_API_KEY')
    developer =Developer(requirement=requirement,root_dir=directory,openai_api_key=api_key)
    developer.develop()